Package: R453Plus1Toolbox
Type: Package
Title: A package for importing and analyzing data from Roche's Genome
        Sequencer System
Version: 1.63.2
Date: 2026-07-01
Author: Hans-Ulrich Klein, Christoph Bartenhagen, Christian Ruckert
Maintainer: Hans-Ulrich Klein <h.klein@uni-muenster.de>
Depends: R (>= 2.12.0), methods, VariantAnnotation (>= 1.25.11),
        Biostrings (>= 2.47.6), pwalign, Biobase
Imports: utils, grDevices, graphics, stats, tools, xtable, R2HTML,
        TeachingDemos, BiocGenerics, S4Vectors (>= 0.17.25), IRanges
        (>= 2.13.12), XVector, GenomicRanges (>= 1.31.8),
        SummarizedExperiment, biomaRt, BSgenome (>= 1.47.3), Rsamtools,
        ShortRead (>= 1.37.1)
Suggests: rtracklayer, BSgenome.Hsapiens.UCSC.hg19,
        BSgenome.Scerevisiae.UCSC.sacCer2
Description: The R453Plus1 Toolbox comprises useful functions for the
        analysis of data generated by Roche's 454 sequencing platform.
        It adds functions for quality assurance as well as for
        annotation and visualization of detected variants,
        complementing the software tools shipped by Roche with their
        product. Further, a pipeline for the detection of structural
        variants is provided.
License: LGPL-3
biocViews: Sequencing, Infrastructure, DataImport, DataRepresentation,
        Visualization, QualityControl, ReportWriting
git_url: https://git.bioconductor.org/packages/R453Plus1Toolbox
git_branch: devel
git_last_commit: c022d30
git_last_commit_date: 2026-07-01
Repository: Bioconductor 3.24
Date/Publication: 2026-07-03
NeedsCompilation: yes
Packaged: 2026-07-03 21:23:39 UTC; biocbuild
Built: R 4.6.1; aarch64-apple-darwin23; 2026-07-04 03:48:19 UTC; unix
