\name{getAminoAbbr}
\alias{getAminoAbbr}
\title{Get amino acid abbreviations}
\description{This function returns a table with amino acid names as first column and the
  according abbreviations as row names.}
\usage{getAminoAbbr()}

